--------------------------------------------------------------------------------
-- CERN (BE-CO-HT)
-- Top level entity of CONV-TTL-BLO V1
-- http://www.ohwr.org/projects/conv-ttl-blo
--------------------------------------------------------------------------------
--
-- unit name: image1_top.vhd
--
-- author: Carlos Gil Soriano (gilsoriano@gmail.com)
--
-- date: 23-01-2013
--
-- version: 1.0
--
-- description: Package for top entity of CONV-TTL-BLO V1
--
-- dependencies:
--
--------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
--------------------------------------------------------------------------------
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--------------------------------------------------------------------------------


library IEEE;
library work;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use IEEE.MATH_REAL.ALL;

--! Packages from IP cores
use work.wishbone_pkg.ALL;
use work.i2c_slave_pkg.ALL;
use work.m25p32_pkg.ALL;
use work.multiboot_pkg.ALL;

package image1_pkg is

   --! Blocking repetition parameters
   constant c_NUMBER_OF_CHANNELS  : NATURAL := 6;

   --! Pulse repetition constants
   constant c_OUTPUT_PULSE_LENGTH : TIME    := 1000 ns;
   constant c_LED_BLINKING_LENGTH : TIME    := (10**6)*250 ns;

   constant c_PLL_IN_PERIOD    : TIME    := 8 ns;
   constant c_CLKIN_PERIOD     : REAL    := REAL(c_PLL_IN_PERIOD/(1 ns));

   constant c_CLKFBOUT_MULT    : NATURAL := 8;
   constant c_CLKOUTA_DIVIDE   : NATURAL := 20;
   constant c_CLKOUTB_DIVIDE   : NATURAL := 5;

   --! As TIME type is derived from INTEGER, please don't let fractional parts
   constant c_WB_CLK_PERIOD    : TIME    := (c_CLKOUTA_DIVIDE*c_PLL_IN_PERIOD)/c_CLKFBOUT_MULT;
   constant c_CLKB_PERIOD      : TIME    := (c_CLKOUTB_DIVIDE*c_PLL_IN_PERIOD)/c_CLKFBOUT_MULT;

   constant c_RST_TIME         : TIME    := 250 ns;
   constant c_RST_PLL_CLKS     : NATURAL := c_RST_TIME/c_PLL_IN_PERIOD;
   constant c_RST_A_CLKS       : NATURAL := c_RST_TIME/c_WB_CLK_PERIOD;
   constant c_RST_B_CLKS       : NATURAL := c_RST_TIME/c_CLKB_PERIOD;

--   component basic_trigger_top
--   generic(g_NUMBER_OF_CHANNELS   : NATURAL := 6;
--           g_CLK_PERIOD           : TIME    := 20 ns;
--           g_OUTPUT_PULSE_LENGTH  : TIME    := 1000 ns;
--           g_LED_BLINKING_LENGTH  : TIME    := (10**6)*250 ns);
--   port (clk_i          : in  STD_LOGIC;
--         rst_i          : in  STD_LOGIC;
--
--         led_ttl_o      : out STD_LOGIC;
--
--         fpga_o_en      : out STD_LOGIC;
--         fpga_o_ttl_en  : out STD_LOGIC;
--         fpga_o_inv_en  : out STD_LOGIC;
--         fpga_o_blo_en  : out STD_LOGIC;
--
--         level_i        : in  STD_LOGIC;
--         switch_i       : in  STD_LOGIC;   --! General enable
--         manual_rst_n_o : out STD_LOGIC;   --! It allows power sequencing of the
--                                           --! 24V rail after a security given
--                                           --! delay
--         pulse_i_front  : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
--         pulse_o_front  : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
--
--         pulse_i_rear   : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
--         pulse_o_rear   : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
--
--         led_o_front    : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
--         led_o_rear     : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
--
--         inv_i          : in  STD_LOGIC_VECTOR(4 downto 1);
--         inv_o          : out STD_LOGIC_VECTOR(4 downto 1));
--   end component;


--   component i2c_slave_top
--   generic(g_WB_CLK_PERIOD : TIME := c_WB_CLK_PERIOD); -- Specify in ns
--   port(sda_oen          : out STD_LOGIC;
--        sda_i            : in  STD_LOGIC;
--        sda_o            : out STD_LOGIC;
--        scl_oen          : out STD_LOGIC;
--        scl_i            : in  STD_LOGIC;
--        scl_o            : out STD_LOGIC;
--
--        wb_clk_i         : in  STD_LOGIC;
--        wb_rst_i         : in  STD_LOGIC;
--
--        wb_master_stb_o  : out STD_LOGIC;
--        wb_master_cyc_o  : out STD_LOGIC;
--        wb_master_sel_o  : out STD_LOGIC_VECTOR(3  downto 0);
--        wb_master_we_o   : out STD_LOGIC;
--        wb_master_data_i : in  STD_LOGIC_VECTOR(31 downto 0);
--        wb_master_data_o : out STD_LOGIC_VECTOR(31 downto 0);
--        wb_master_addr_o : out STD_LOGIC_VECTOR(15  downto 0);
--        wb_master_ack_i  : in  STD_LOGIC;
--        wb_master_rty_i  : in  STD_LOGIC;
--        wb_master_err_i  : in  STD_LOGIC;
--
--        wb_slave_stb_i   : in  STD_LOGIC;
--        wb_slave_cyc_i   : in  STD_LOGIC;
--        wb_slave_sel_i   : in  STD_LOGIC_VECTOR(3  downto 0);
--        wb_slave_we_i    : in  STD_LOGIC;
--        wb_slave_data_i  : in  STD_LOGIC_VECTOR(31 downto 0);
--        wb_slave_data_o  : out STD_LOGIC_VECTOR(31 downto 0);
--        wb_slave_addr_i  : in  STD_LOGIC_VECTOR(3  downto 0);
--        wb_slave_ack_o   : out STD_LOGIC;
--        wb_slave_rty_o   : out STD_LOGIC;
--        wb_slave_err_o   : out STD_LOGIC;
--
--        pf_wb_addr_o     : out STD_LOGIC;
--        rd_done_o        : out STD_LOGIC;
--        wr_done_o        : out STD_LOGIC;
--        i2c_addr_i       : in  STD_LOGIC_VECTOR(6  downto 0)
--        );
--   end component;

--   component m25p32_top
--   generic(g_WB_ADDR_LENGTH : NATURAL := c_WORDS_PER_PAGE_BITS + 1);
--   port(wb_rst_i       : in  STD_LOGIC;
--        wb_clk_i       : in  STD_LOGIC;
--
--        wb_we_i        : in  STD_LOGIC;
--        wb_stb_i       : in  STD_LOGIC;
--        wb_cyc_i       : in  STD_LOGIC;
--        wb_sel_i       : in  STD_LOGIC_VECTOR (3  downto 0);
--        wb_data_i      : in  STD_LOGIC_VECTOR (31 downto 0);
--        wb_data_o      : out STD_LOGIC_VECTOR (31 downto 0);
--        wb_addr_i      : in  STD_LOGIC_VECTOR (g_WB_ADDR_LENGTH - 1 downto 0);
--        wb_ack_o       : out STD_LOGIC;
--        wb_rty_o       : out STD_LOGIC;
--        wb_err_o       : out STD_LOGIC;
--
--        miso_word_rcv  : out STD_LOGIC;
--        op_finished_o  : out STD_LOGIC;
--
--        prom_mosi_o    : out STD_LOGIC;
--        prom_cclk_o    : out STD_LOGIC;
--        prom_cs0_b_n_o : out STD_LOGIC;
--        prom_din_i     : in  STD_LOGIC);
--   end component;
--
--   component multiboot_top
--   generic(g_MBA_addr : UNSIGNED (23 downto 0) := UNSIGNED(c_MBA_map_addr);
--           g_GBA_addr : UNSIGNED (23 downto 0) := UNSIGNED(c_GBA_map_addr);
--           --! This is a vendor-dependant SPI opcode.
--           --! Pleasem take a look to the flash memory manual and put here
--           --! the op-code/instruction-code for a read operation over SPI
--           --! i.e. for m25p32 memory that corresponds to X"03"
--           --! Set it up accordingly.
--           g_READ_SPI_OPCODE : STD_LOGIC_VECTOR(7 downto 0) := X"03");
--   port(wb_rst_i   : in   STD_LOGIC;
--        wb_clk_i   : in   STD_LOGIC;
--        wb_we_i    : in   STD_LOGIC;
--        wb_stb_i   : in   STD_LOGIC;
--        wb_cyc_i   : in   STD_LOGIC;
--        wb_sel_i   : in   STD_LOGIC_VECTOR (3  downto 0);
--        wb_data_i  : in   STD_LOGIC_VECTOR (31 downto 0);
--        wb_data_o  : out  STD_LOGIC_VECTOR (31 downto 0);
--        wb_addr_i  : in   STD_LOGIC_VECTOR (3  downto 0);
--        wb_ack_o   : out  STD_LOGIC;
--        wb_rty_o   : out  STD_LOGIC;
--        wb_err_o   : out  STD_LOGIC);
--   end component;

--   component rtm_detector
--   generic(g_identifier_RTMM : t_RTMM := RTMM_V1;
--           g_identifier_RTMP : t_RTMP := RTMP_BLOCKING_V1);
--   port (RTMM_i     : in STD_LOGIC_VECTOR(2 downto 0);
--         RTMP_i     : in STD_LOGIC_VECTOR(2 downto 0);
--         ok_RTMM_o  : out STD_LOGIC;
--         ok_RTMP_o  : out STD_LOGIC);
--   end component;


   --   function check_sys_cfg return BOOLEAN;
   function f_RENESAS_I2C_ADDRESSING(VME_slot : UNSIGNED(4 downto 0))
                                                       return STD_LOGIC_VECTOR;
end image1_pkg;

package body image1_pkg is

--   function check_sys_cfg return BOOLEAN is
--      variable v_real_tmp  : REAL;
--      variable v_time_tmp  : TIME;
--      variable v_return    : BOOLEAN;
--   begin
--      v_real_tmp := c_CLKIN_PERIOD;
--      assert (v_real_tmp = 0.0)  report REAL'image(v_real_tmp) severity Failure;
--      v_time_tmp := c_WB_CLK_PERIOD;
--      assert (v_time_tmp = 0 ns) report TIME'image(v_time_tmp) severity Failure;
--      v_time_tmp := c_CLKB_PERIOD;
--      assert (v_time_tmp = 0 ns) report TIME'image(v_time_tmp) severity Failure;
--      return TRUE;
--   end check_sys_cfg;


   --! @brief           Translation function for i2c_addressing
   --!                  The specification of this criteria can be found in
   --!                  Samuel's document "Specification. Access to board data
   --!                  using SNMP and I2C" sent to Mihai Savu.
   --! @param VME_slot  VME_slot (FPGA_GA[x] pins) in which the board is
   --!                  inserted
   function f_RENESAS_I2C_ADDRESSING(VME_slot : UNSIGNED(4 downto 0))
                                                    return STD_LOGIC_VECTOR is
      variable v_tmp   : NATURAL;
      variable v_return  : STD_LOGIC_VECTOR(6 downto 0);
   begin
      v_tmp := 95 - to_integer(VME_slot);
      v_return := STD_LOGIC_VECTOR(to_unsigned(v_tmp, 7));
      return v_return;
   end function;

end image1_pkg;
