--------------------------------------------------------------------------------
-- CERN (BE-CO-HT)
-- Top level entity of CONV-TTL-BLO V1
-- http://www.ohwr.org/projects/conv-ttl-blo
--------------------------------------------------------------------------------
--
-- unit name: image1_wrappers_pkg.vhd
--
-- author: Carlos Gil Soriano (gilsoriano@gmail.com)
--
-- date: 23-01-2013
--
-- version: 1.0
--
-- description: Package for the wrappers used in image1_core.vhd of
--              CONV-TTL-BLO V2
--
-- dependencies:
--
--------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
--------------------------------------------------------------------------------
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--------------------------------------------------------------------------------


library IEEE;
library work;

use IEEE.STD_LOGIC_1164.ALL;

package image1_wrappers_pkg is

   --! Wrappers for image1_top FPGA pinouts
   type t_led_array_o is
      record
         CTRL0            : STD_LOGIC;
         CTRL0_OEN        : STD_LOGIC;
         CTRL1            : STD_LOGIC;
         CTRL1_OEN        : STD_LOGIC;
         MULTICAST_2_0    : STD_LOGIC;
         MULTICAST_3_1    : STD_LOGIC;
         WR_GMT_TTL_TTLN  : STD_LOGIC;
         WR_LINK_SYSERROR : STD_LOGIC;
         WR_OK_SYSPW      : STD_LOGIC;
         WR_OWNADDR_I2C   : STD_LOGIC;
      end record;

   type t_i2c_slave_i is
      record
         SCL_I   : STD_LOGIC;
         SDA_I   : STD_LOGIC;
      end record;

   type t_i2c_slave_o is
      record
         SCL_O   : STD_LOGIC;
         SCL_OE  : STD_LOGIC;
         SDA_O   : STD_LOGIC;
         SDA_OE  : STD_LOGIC;
      end record;

   type t_spi_master_i is
      record
         DIN     : STD_LOGIC;
      end record;

   type t_spi_master_o is
      record
         CCLK    : STD_LOGIC;
         CSO_B_N : STD_LOGIC;
         MOSI    : STD_LOGIC;
      end record;

   type t_rtm_i is
      record
         RTMM_N  : STD_LOGIC_VECTOR(2 downto 0);
         RTMP_N  : STD_LOGIC_VECTOR(2 downto 0);
      end record;


      constant c_led_array_default : t_led_array_o := (CTRL0            => '0',
                                                       CTRL0_OEN        => '0',
                                                       CTRL1            => '0',
                                                       CTRL1_OEN        => '0',
                                                       MULTICAST_2_0    => '0',
                                                       MULTICAST_3_1    => '0',
                                                       WR_GMT_TTL_TTLN  => '0',
                                                       WR_LINK_SYSERROR => '0',
                                                       WR_OK_SYSPW      => '0',
                                                       WR_OWNADDR_I2C =>  '0');

end image1_wrappers_pkg;

package body image1_wrappers_pkg is
end image1_wrappers_pkg;
