library IEEE;
library work;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.multiboot_pkg.ALL;


entity multiboot_top is
   generic(g_MBA_addr : UNSIGNED (23 downto 0) := UNSIGNED(c_MBA_map_addr);
           g_GBA_addr : UNSIGNED (23 downto 0) := UNSIGNED(c_GBA_map_addr);
           --! This is a vendor-dependant SPI opcode.
           --! Pleasem take a look to the flash memory manual and put here
           --! the op-code/instruction-code for a read operation over SPI
           --! i.e. for m25p32 memory that corresponds to X"03"
           --! Set it up accordingly.
           g_READ_SPI_OPCODE : STD_LOGIC_VECTOR(7 downto 0) := X"03");
   port(wb_rst_i   : in   STD_LOGIC;
        wb_clk     : in   STD_LOGIC;
        wb_we_i    : in   STD_LOGIC;
        wb_stb_i   : in   STD_LOGIC;
        wb_cyc_i   : in   STD_LOGIC;
        wb_sel_i   : in   STD_LOGIC_VECTOR (3  downto 0);
        wb_data_i  : in   STD_LOGIC_VECTOR (31 downto 0);
        wb_data_o  : out  STD_LOGIC_VECTOR (31 downto 0);
        wb_addr_i  : in   STD_LOGIC_VECTOR (3  downto 0);
        wb_ack_o   : out  STD_LOGIC;
        wb_rty_o   : out  STD_LOGIC;
        wb_err_o   : out  STD_LOGIC);
end multiboot_top;


architecture Behavioral of multiboot_top is

signal s_CTR0     : r_CTR0;
signal s_CTR1     : r_CTR1;
signal s_STAT     : r_STAT;
signal s_MBA      : r_BAR;
signal s_GBA      : r_BAR;
signal s_MBA_addr : STD_LOGIC_VECTOR(23 downto 0);
signal s_GBA_addr : STD_LOGIC_VECTOR(23 downto 0);

begin

  multiboot_regs_inst : multiboot_regs
  generic map (g_MBA_addr        => g_MBA_addr,
               g_GBA_addr        => g_GBA_addr,
               g_READ_SPI_OPCODE => g_READ_SPI_OPCODE)
  port map(wb_rst_i     => wb_rst_i,
           wb_clk       => wb_clk,
           wb_we_i      => wb_we_i,
           wb_stb_i     => wb_stb_i,
           wb_cyc_i     => wb_cyc_i,
           wb_sel_i     => wb_sel_i,
           wb_data_i    => wb_data_i,
           wb_data_o    => wb_data_o,
           wb_addr_i    => wb_addr_i,
           wb_ack_o     => wb_ack_o,
           wb_rty_o     => wb_rty_o,
           wb_err_o     => wb_err_o,
           CTR0_o       => s_CTR0,
           CTR1_i       => s_CTR1,
           STAT_i       => s_STAT,
           MBA_o        => s_MBA,
           GBA_o        => s_GBA,
           MBA_ICAP_i   => s_MBA_addr,
           GBA_ICAP_i   => s_GBA_addr);

  multiboot_core_inst: multiboot_core
  port map(clk          =>  wb_clk,
           rst          =>  wb_rst_i,
           CTR0_i       => s_CTR0,
           CTR1_o       => s_CTR1,
           STAT_o       => s_STAT,
           MBA_i        => s_MBA,
           GBA_i        => s_GBA,
           MBA_ICAP_o   => s_MBA_addr,
           GBA_ICAP_o   => s_GBA_addr);

end Behavioral;
